self.__BUILD_MANIFEST={__rewrites:{beforeFiles:[],afterFiles:[],fallback:[]},"/":["static/chunks/2852872c-eeb7556500d719ed658b.js","static/chunks/4ad82c5e-66a2dbe5383a19d54820.js","static/chunks/75fc9c18-ba4bdb366cbba6dfcf56.js","static/chunks/915-9631ad5496d39b49fd89.js","static/chunks/pages/index-de249c4044d72cf06d23.js"],"/_error":["static/chunks/pages/_error-ea939aab753d9e9db3bd.js"],sortedPages:["/","/_app","/_error"]},self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();